#include <iostream>  
#include <fstream>  
#include <sstream>   // String stream
using namespace std;

#define MAXPPL 10
#define MAXSTR 200

struct person_record_t
{
	char name[MAXSTR];
	int age;
};

void print_person_ptr(person_record_t *p)
{
	cout << "Name: " << p->name << endl;
	cout << "Age: " << p->age << endl;
}



void celebrate_birthday_ptr(person_record_t *ptr)
{
	ptr->age++; // Is just a pretty version of (*ptr).age++;
		    // this is syntatical sugar

	// Warning: *ptr.field is actually this *(ptr.field)
}

void read_person(char *inputline, person_record_t &output)
{
	/* Input: takes a string, from our file, called inputline.
	 * Output: produces a person_record_t
	 * Work: splits the input line */

	stringstream ss(inputline);

	ss >> output.name;
	ss >> output.age;
	
}

int main(int argc, char **argv)
{
	/*	
	person_record_t p = {"Alberto", 13};
	print_person(p);
	person_record_t q = {"Jane", 33};
	print_person(q);
	*/


	if (argc != 2) {
		cerr << "Expecting exactly one argument for the file to read!" << endl;
		return 1;
	}


	// cout << "file to open is " << argv[1] << endl;
	ifstream ifs; // stream to read the file from
	ifs.open(argv[1], ios_base::in); 

	if (!ifs) {
		cerr << "Unable to read file \"" << argv[1] << "\"" << endl;
		return 1;

	}


	person_record_t people[MAXPPL];
	char line[MAXSTR];	

	int total_people = 0;
	while (ifs.getline(line, MAXSTR))
	{
		// cout << "Line " << total_people << ": '" << line << "'" << endl;
		read_person(line, people[total_people]);
		total_people++;
		if (total_people >= MAXPPL)
		{
			cerr << "Sorry, your input is too large." << endl;
			ifs.close(); 
			return 1;
		}
	}

	ifs.close();


	// Next task is to make an age ordered index
	// <type> <name> [size]
	person_record_t* age_idx[total_people];
	
	for (int i = 0; i < total_people; i++)
		age_idx[i] = &people[total_people-i-1]; // This index is to put them in reverse order
		// age_idx[i] = &people[i]; // This index is to put them in the same order

	cout << "==================================================" << endl;
	for (int i = 0; i < total_people; i++)
		print_person_ptr(age_idx[i]);

	int count_j = 0;
	int count_not_j = 0;
	person_record_t* starts_with_j[total_people];
	person_record_t* starts_with_not_j[total_people];
	for (int i = 0; i < total_people; i++)
	{
		if (people[i].name[0] == 'J')
		{
			starts_with_j[count_j++] = &people[i];
		}
		else
		{
			starts_with_not_j[count_not_j++] = &people[i];
		}
	}

	cout << "==================================================" << endl;
	//for (int i = 0; i < total_people; i++)  // This will segfault because some pointers are uninitialized pointer
	for (int i = 0; i < count_j; i++)
		print_person_ptr(starts_with_j[i]);


	
	return 0;
}


